<?php

/**
 * NAT firewall controller.
 *
 * @category   apps
 * @package    nat-firewall
 * @subpackage controllers
 * @author     ClearFoundation <developer@clearfoundation.com>
 * @copyright  2011 ClearFoundation
 * @license    http://www.gnu.org/copyleft/gpl.html GNU General Public License version 3 or later
 * @link       http://www.clearfoundation.com/docs/developer/apps/nat_firewall/
 */

///////////////////////////////////////////////////////////////////////////////
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

use \clearos\apps\network\Role as Role;
use \clearos\apps\nat_firewall\One_To_One_NAT as One_To_One_NAT;

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * NAT firewall controller.
 *
 * @category   apps
 * @package    nat-firewall
 * @subpackage controllers
 * @author     ClearFoundation <developer@clearfoundation.com>
 * @copyright  2011 ClearFoundation
 * @license    http://www.gnu.org/copyleft/gpl.html GNU General Public License version 3 or later
 * @link       http://www.clearfoundation.com/docs/developer/apps/nat_firewall/
 */

class Nat_Firewall extends ClearOS_Controller
{
    /**
     * NAT firewall overview.
     *
     * @return view
     */

    function index()
    {
        // Load libraries
        //---------------

        $this->lang->load('nat_firewall');
        $this->load->library('nat_firewall/One_To_One_NAT');
        $this->load->library('network/Iface_Manager');
        $this->load->library('network/Network');

        // Sanity check - make sure there is a external interface configured
        //-------------

        $ifaces = $this->iface_manager->get_external_interfaces();

        if (count($ifaces) < 1)
            $this->page->set_message(lang('nat_firewall_no_external_nic'), 'warning');

        // Load view data
        //---------------

        try {
            $data['nat_rules'] = $this->one_to_one_nat->get_nat_rules();
            $data['network_mode'] = $this->network->get_mode();
            $data['panic'] = $this->one_to_one_nat->is_panic();
        } catch (Exception $e) {
            $this->page->view_exception($e);
            return;
        }
 
        // Load views
        //-----------

        $options['type'] = MY_Page::TYPE_WIDE_CONFIGURATION;

        $this->page->view_form('nat_firewall/summary', $data, lang('nat_firewall_app_name'), $options);
    }

    /**
     * Add rule.
     *
     * @return view
     */

    function add()
    {
        // Load libraries
        //---------------

        $this->load->library('nat_firewall/One_To_One_NAT');
        $this->load->library('network/Iface_Manager');
        $this->lang->load('nat_firewall');

        // Set validation rules
        //---------------------

        $this->form_validation->set_policy('nickname', 'nat_firewall/One_To_One_NAT', 'validate_name', TRUE);
        $this->form_validation->set_policy('interface', 'nat_firewall/One_To_One_NAT', 'validate_interface', TRUE);
        $this->form_validation->set_policy('private_ip', 'nat_firewall/One_To_One_NAT', 'validate_ip', TRUE);
        $this->form_validation->set_policy('public_ip', 'nat_firewall/One_To_One_NAT', 'validate_ip', TRUE);

        if ($this->input->post('all') != 'on') {
            $this->form_validation->set_policy('protocol', 'nat_firewall/One_To_One_NAT', 'validate_protocol', TRUE);
            $this->form_validation->set_policy('port', 'nat_firewall/One_To_One_NAT', 'validate_port');
        }

        $form_ok = $this->form_validation->run();

        // Handle form submit
        //-------------------

        if (($this->input->post('submit') && $form_ok)) {
            try {
                if ($this->input->post('all') == 'on') {
                    $protocol = One_To_One_NAT::PROTOCOL_ALL;
                    $port = One_To_One_NAT::CONSTANT_ALL_PORTS;
                } else {
                    $protocol = $this->input->post('protocol');
                    $port = $this->input->post('port');
                }

                $this->one_to_one_nat->add(
                    $this->input->post('nickname'),
                    $this->input->post('public_ip'),
                    $this->input->post('private_ip'),
                    $protocol,
                    $port,
                    $this->input->post('interface')
                );

                $this->page->set_status_added();
                redirect('/nat_firewall');
            } catch (Exception $e) {
                $this->page->view_exception($e);
                return;
            }
        }

        // Load view data
        //---------------

        try {
            $data['protocols'] = $this->one_to_one_nat->get_basic_protocols();
            $data['interfaces'] = $this->iface_manager->get_external_interfaces();
        } catch (Exception $e) {
            $this->page->view_exception($e);
            return;
        }
 
        // Load the views
        //---------------

        $this->page->view_form('nat_firewall/add', $data, lang('base_add'));
    }

    /**
     * Delete rule.
     *
     * @param string $name      nickname
     * @param string $wan_ip    IP address
     * @param string $lan_ip    IP address
     * @param string $protocol  protocol
     * @param string $port      port
     * @param string $interface port
     *
     * @return view
     */

    function delete($name, $wan_ip, $lan_ip, $protocol, $port, $interface)
    {
        $this->lang->load('nat_firewall');
        $confirm_uri = '/app/nat_firewall/destroy/' . $wan_ip . '/' . $lan_ip . '/' . $protocol . '/' . $port . '/' . $interface;
        $cancel_uri = '/app/nat_firewall';
        $items = array($name . ' (' . $wan_ip . ' > ' . $lan_ip . ') ');

        $this->page->view_confirm_delete($confirm_uri, $cancel_uri, $items);
    }

    /**
     * Destroys rule.
     *
     * @param string $wan_ip    IP address
     * @param string $lan_ip    IP address
     * @param string $protocol  protocol
     * @param string $port      port
     * @param string $interface port
     *
     * @return view
     */

    function destroy($wan_ip, $lan_ip, $protocol, $port, $interface)
    {
        try {
            $this->load->library('nat_firewall/One_To_One_NAT');

            $this->one_to_one_nat->delete($wan_ip, $lan_ip, $protocol, ($port ? $port : 0), $interface);

            $this->page->set_status_deleted();
            redirect('/nat_firewall');
        } catch (Exception $e) {
            $this->page->view_exception($e);
            return;
        }
    }

    /**
     * Disables rule.
     *
     * @param string $name      nickname
     * @param string $wan_ip    IP address
     * @param string $lan_ip    IP address
     * @param string $protocol  protocol
     * @param string $port      port
     * @param string $interface port
     *
     * @return view
     */

    function disable($name, $wan_ip, $lan_ip, $protocol, $port, $interface)
    {
        try {
            $this->load->library('nat_firewall/One_To_One_NAT');

            $this->one_to_one_nat->toggle_enable(FALSE, $wan_ip, $lan_ip, $protocol, $port, $interface);

            $this->page->set_status_disabled();
            redirect('/nat_firewall');
        } catch (Exception $e) {
            $this->page->view_exception($e);
            return;
        }
    }

    /**
     * Enables rule.
     *
     * @param string $name      nickname
     * @param string $wan_ip    IP address
     * @param string $lan_ip    IP address
     * @param string $protocol  protocol
     * @param string $port      port
     * @param string $interface port
     *
     * @return view
     */

    function enable($name, $wan_ip, $lan_ip, $protocol, $port, $interface)
    {
        try {
            $this->load->library('nat_firewall/One_To_One_NAT');

            $this->one_to_one_nat->toggle_enable(TRUE, $wan_ip, $lan_ip, $protocol, $port, $interface);

            $this->page->set_status_enabled();
            redirect('/nat_firewall');
        } catch (Exception $e) {
            $this->page->view_exception($e);
            return;
        }
    }
}
